'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = createFluentInterface;

var _utils = require('./utils');

var _options = require('./options');

var _constants = require('./constants');

var ALIAS_IDX = 0;
var VALUE_IDX = 1;
var TYPE_IDX = 2;
var DEPS_IDX = 3;

function createChildContextFactory(conf) {
	return function (context) {
		return context.createChild(conf);
	};
}

function createExporter(contextFactory, exportAlias) {
	return function (configure) {
		return contextFactory(configure).get(exportAlias);
	};
}

function createFluentInterface(contribute) {
	var pending = [];

	return {
		map: function map(alias) {
			this.done();
			pending.push(alias);

			return {
				to: function to(value) {

					pending.push(value);

					return {
						as: function as() {
							for (var _len = arguments.length, flags = Array(_len), _key = 0; _key < _len; _key++) {
								flags[_key] = arguments[_key];
							}

							var mask = (0, _utils.generateMask)(flags);

							if (mask === _options.BLUEPRINT) {
								// TODO: enforve value to be a function
								flags = [_options.PROVIDER];
								pending[VALUE_IDX] = createChildContextFactory(pending[VALUE_IDX]);
								pending.push((0, _utils.generateMask)(flags));
								pending.push([_constants.CONTEXT_ALIAS]);

								return {
									exports: function exports(alias) {
										pending[VALUE_IDX] = createExporter(pending[VALUE_IDX], alias);
									}
								};
							}

							pending.push(mask);

							return {
								injecting: function injecting() {
									for (var _len2 = arguments.length, deps = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
										deps[_key2] = arguments[_key2];
									}

									pending.push(deps);
								}
							};
						}
					};
				}
			};
		},

		done: function done() {
			var deps = pending[DEPS_IDX] || [];

			if (deps.length === 0) {
				deps = (0, _utils.extractImplicitDependencies)(pending[VALUE_IDX]);
			}

			if (pending.length > 2) {
				contribute(pending[ALIAS_IDX], pending[VALUE_IDX], pending[TYPE_IDX], deps);
				pending = [];
			}
		}
	};
}

module.exports = exports['default'];