'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = createPropertyMapper;
var SUPPORT_CODE_INTROSPECTION = typeof Object.getOwnPropertyDescriptor === 'function';
var hasOwnProperty = Object.prototype.hasOwnProperty;

function lookupForDescriptor(_x, _x2) {
	var _again = true;

	_function: while (_again) {
		var object = _x,
		    propertyName = _x2;
		_again = false;

		if (object === null) {
			return undefined;
		}
		if (hasOwnProperty.call(object, propertyName)) {
			return Object.getOwnPropertyDescriptor(object, propertyName);
		}
		_x = Object.getPrototypeOf(object);
		_x2 = propertyName;
		_again = true;
		continue _function;
	}
}

function createGetter(instance, propertyName) {
	return function () {
		if (arguments.length > 0) {
			throw new Error('You cannot use a property accessor to mutate a read-only property.');
		}
		return instance[propertyName];
	};
}

function createSetter(instance, propertyName) {
	return function (value) {
		if (arguments.length === 0) {
			throw new Error('You cannot use a property mutator to read a write-only property.');
		}
		instance[propertyName] = value;
	};
}

function createSetterGetter(instance, propertyName) {
	return function (value) {
		if (arguments.length === 0) {
			return instance[propertyName];
		}
		instance[propertyName] = value;
	};
}

function createPropertyProxy(instance, propertyName) {
	if (SUPPORT_CODE_INTROSPECTION) {
		var descriptor = lookupForDescriptor(instance, propertyName);

		if (typeof descriptor.get === 'function' && typeof descriptor.set === 'function') {
			return createSetterGetter(instance, propertyName);
		}
		if (typeof descriptor.get === 'function') {
			return createGetter(instance, propertyName);
		}
		if (typeof descriptor.set === 'function') {
			return createSetter(instance, propertyName);
		}
	}
	return createSetterGetter(instance, propertyName);
}

function extractProperty(instance, propertyName) {
	if (typeof instance[propertyName] === 'function') {
		return instance[propertyName].bind(instance);
	}
	return createPropertyProxy(instance, propertyName);
}

function createPropertyMapper(createMapping) {
	for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
		args[_key - 1] = arguments[_key];
	}

	var mapping = createMapping.apply(null, args);

	return Object.create(mapping, {
		get: {
			value: function value(propertyName) {
				var instance = mapping.get();

				if (propertyName) {
					return extractProperty(instance, propertyName);
				}
				return instance;
			}
		}
	});
}

module.exports = exports['default'];