'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = createSingletonMappingFactory;

function createSingletonMappingFactory(create) {
	var isTransient = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

	return function (value, resolveDeps, releaseDeps) {
		var handlers = {};
		var count = 0;
		var instance;

		return {
			get: function get() {
				if (typeof instance === 'undefined') {
					instance = create.call(this, value, resolveDeps());
				}
				count++;
				return instance;
			},
			release: function release() {
				count--;
				if (count == 0 && isTransient) {
					this.dispose();
				}
			},
			dispose: function dispose() {
				if (instance) {
					releaseDeps();
					if (typeof instance.dispose === 'function') {
						instance.dispose();
					}
					instance = undefined;
					count = 0;
				}
			}
		};
	};
}

module.exports = exports['default'];