'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.identity = identity;
exports.invoke = invoke;
exports.partial = partial;
exports.instanciate = instanciate;
exports.generateMask = generateMask;
exports.isEagerSingleton = isEagerSingleton;
exports.noop = noop;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _extractImplicitDependencies = require('./extractImplicitDependencies');

var _extractImplicitDependencies2 = _interopRequireDefault(_extractImplicitDependencies);

var _options = require('./options');

var EAGER_SINGLETON_PROVIDER = generateMask([_options.EAGER, _options.SINGLETON, _options.PROVIDER]);
var EAGER_SINGLETON_CONSTRUCTOR = generateMask([_options.EAGER, _options.SINGLETON, _options.CONSTRUCTOR]);

function identity(value) {
	return value;
}

function invoke(fn, deps) {
	return fn.apply(null, deps);
}

function partial(func, deps) {
	return function () {
		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return func.apply(this, deps.concat(args));
	};
}

function instanciate(Ctor, deps) {
	var instance = Object.create(Ctor.prototype);

	Ctor.apply(instance, deps);
	return instance;
}

function generateMask(flags) {
	return flags.reduce(function (prev, curr) {
		return prev | curr;
	}, 0);
}

function isEagerSingleton(type) {
	return [EAGER_SINGLETON_PROVIDER, EAGER_SINGLETON_CONSTRUCTOR].indexOf(type) > -1;
}

function noop() {}

exports.extractImplicitDependencies = _extractImplicitDependencies2['default'];